/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package EDU.rutgers.CORBA.GIOP.actions;

import EDU.rutgers.act.protocol.Message;
import EDU.rutgers.act.endpoint.Endpoint;

import EDU.rutgers.CORBA.GIOP.messages.*;
import EDU.rutgers.CORBA.IIOP.ProfileBody;
import EDU.rutgers.CORBA.IOP.IOR;

import EDU.rutgers.act.protocol.FSMAction;

public final class GIOPClientLocateAction implements FSMAction
{
    public int run(Message previous, Message current, Message[] next) {
	    next[0] = null;
        if(current instanceof GIOPLocateReplyMessage) {
            GIOPLocateReplyMessage locateReply = (GIOPLocateReplyMessage)current;
            switch (locateReply.locate_status_.value()) {
                case LocateStatusType._OBJECT_FORWARD : {
	                if(previous instanceof GIOPLocateRequestMessage) {
                        IOR ior = locateReply.getIOR();
                        if (ior == null) {
                            break;
                        }
                        ProfileBody profile = ior.getIIOPProfile();
                        if (profile == null) {
                            break;
                        }
                        Endpoint ep = new Endpoint(profile.host_, profile.port_);

                        GIOPLocateRequestMessage locateRequest = (GIOPLocateRequestMessage)previous;
	                    locateRequest.setEndpoint(ep);

	                    next[0] = locateRequest;
	                    return GIOPAction.LOCATE;
	                }
                    break;
                }
                case LocateStatusType._UNKNOWN_OBJECT : {
	                return FSMAction.ERROR;
                }
                case LocateStatusType._OBJECT_HERE : {
                    return FSMAction.EXIT;
                }
            }
        }
        return FSMAction.ERROR;
    }
}
