/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package EDU.rutgers.CORBA.GIOP.messages;

import ORG.omg.CORBA.BAD_PARAM;

/**
 * GIOP LocateReply Mesage LocateStatusType
 * [CORBA 2.0 Specification 12-22]
 *
 * @since ACT 0.1
 */
public final class LocateStatusType
{
    /**
     * UNKNOWN_OBJECT type value
     *
     * @since ACT 0.1
     */
    public static final int _UNKNOWN_OBJECT = 0;

    /**
     * OBJECT_HERE type value
     *
     * @since ACT 0.1
     */
    public static final int _OBJECT_HERE = 1;

    /**
     * OBJECT_FORWARD type value
     *
     * @since ACT 0.1
     */
    public static final int _OBJECT_FORWARD = 2;

    /**
     * UNKNOWN_OBJECT type
     *
     * @since ACT 0.1
     */
    public static final LocateStatusType UNKNOWN_OBJECT = 
		new LocateStatusType(_UNKNOWN_OBJECT);

    /**
     * OBJECT_HERE type
     *
     * @since ACT 0.1
     */
    public static final LocateStatusType OBJECT_HERE = 
		new LocateStatusType(_OBJECT_HERE);

    /**
     * OBJECT_FORWARD type
     *
     * @since ACT 0.1
     */
    public static final LocateStatusType OBJECT_FORWARD = 
		new LocateStatusType(_OBJECT_FORWARD);

    private LocateStatusType(int v) {
        value = v;
    }

    private int value;

    /**
     * Retrieve the type's value
     *
     * @return int the type value
     *
     * @since ACT %Version%
     */
    public int value() {
		return value;
    }

    /**
     * From the type integer value, retrieve the LocateStatusType
     *
     * @param int the type integer value
     *
     * @return LocateStatusType the corresponding LocateStatusType
     *
     * @exception BAD_PARAM if the integer value is out of range
     *
     * @see ORG.omg.CORBA.BAD_PARAM
     *
     * @since ACT 0.1
     */
    public static LocateStatusType type(int v)
		throws BAD_PARAM {
        switch(v) {
            default:
                throw new BAD_PARAM("Enum out of range[0...2]" + v);

            case _UNKNOWN_OBJECT:
                return LocateStatusType.UNKNOWN_OBJECT;

            case _OBJECT_HERE:
                return LocateStatusType.OBJECT_HERE;

            case _OBJECT_FORWARD:
                return LocateStatusType.OBJECT_FORWARD;
        }
    }

    /**
     * Returns a string representation of the locate status. 
     *
     * @return String locate status type description
     *
     * @since ACT 0.1
     */
    public String toString() {
		String[] types = {"UNKNOWN_OBJECT",
						 "OBJECT_HERE",
						 "OBJECT_FORWARD",};
		return types[value];
    }
}
