/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package EDU.rutgers.CORBA.GIOP.messages;

import ORG.omg.CORBA.BAD_PARAM;

/**
 * GIOP Mesage Type
 * <p>[CORBA 2.0 Specification 12-16]
 *
 * @since ACT 0.1
 */
public final class MessageType
{
    /**
     * Request type value
     *
     * @since ACT 0.1
     */
    public static final int REQUEST = 0;

    /**
     * Reply type value
     *
     * @since ACT 0.1
     */
    public static final int REPLY = 1;

    /**
     * CancelRequest type value
     * @since ACT 0.1
     */
    public static final int CANCEL_REQUEST = 2;

    /**
     * LocateRequest type value
     *
     * @since ACT 0.1
     */
    public static final int LOCATE_REQUEST = 3;

    /**
     * LocateReply type value
     *
     * @since ACT 0.1
     */
    public static final int LOCATE_REPLY = 4;

    /**
     * CloseConnection type value
     *
     * @since ACT 0.1
     */
    public static final int CLOSE_CONNECTION = 5;

    /**
     * MessageError type value
     *
     * @since ACT 0.1
     */
    public static final int MESSAGE_ERROR = 6;

    /**
     * Client Request message type
     *
     * @since ACT 0.1
     */
    public static final MessageType Request =
		new MessageType(REQUEST);

    /**
     * Server Reply message type
     *
     * @since ACT 0.1
     */
    public static final MessageType Reply = 
		new MessageType(REPLY);

    /**
     * Client CancelRequest message type
     *
     * @since ACT 0.1
     */
    public static final MessageType CancelRequest =
		new MessageType(CANCEL_REQUEST);

    /**
     * Client LocateRequest message type
     *
     * @since ACT 0.1
     */
    public static final MessageType LocateRequest =
		new MessageType(LOCATE_REQUEST);

    /**
     * Server LocateReply message type
     *
     * @since ACT 0.1
     */
    public static final MessageType LocateReply =
		new MessageType(LOCATE_REPLY);

    /**
     * Server closeConnection message type
     *
     * @since ACT 0.1
     */
    public static final MessageType CloseConnection =
		new MessageType(CLOSE_CONNECTION);

    /**
     * Client/Server MessageError message type
     *
     * @since ACT 0.1
     */
    public static final MessageType MessageError =
		new MessageType(MESSAGE_ERROR);

    /**
     * @since ACT 0.1
     */
    private MessageType(int V) {
        value = V; 
    }

    /**
     * Type value
     *
     * @since ACT 0.1
     */
    private int value;

    /**
     * Retrieve the type value
     *
     * @return int the type value
     *
     * @since ACT %Version%
     */
    public int value() {
        return value; 
    }

    /**
     * From the type integer value, retrieve the MessageType
     *
     * @param int the type integer value
     *
     * @return MessageType the corresponding MessageType
     *
     * @exception BAD_PARAM if the integer value is out of range
     *
     * @since ACT 0.1
     */
    public static MessageType type(int v) 
		throws BAD_PARAM {
        switch (v) {
            default:
                throw new BAD_PARAM("Out of range [0...6] :" + v);

            case REQUEST:
                return MessageType.Request; 

            case REPLY:
                return MessageType.Reply; 

            case CANCEL_REQUEST:
                return MessageType.CancelRequest; 

            case LOCATE_REQUEST:
                return MessageType.LocateRequest; 

            case LOCATE_REPLY:
                return MessageType.LocateReply; 

            case CLOSE_CONNECTION:
                return MessageType.CloseConnection; 

            case MESSAGE_ERROR:
                return MessageType.MessageError;
        }
    }

    /**
     * Returns a string representation.
     *
     * @return String description
     *
     * @since ACT 0.1
     */
    public String toString() {
		String[] types = {"Request", 
						  "Reply",	
						  "CancelRequest",
						  "LocateRequest",
						  "LocateReply",
						  "CloseConnection",
						  "MessageError",};		 
		return types[value];
    }
}

