/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package EDU.rutgers.CORBA.GIOP.messages;

import ORG.omg.CORBA.Streamable;

import EDU.rutgers.act.presentation.TypeReader;
import EDU.rutgers.act.presentation.TypeWriter;

/**
 * Version number of the GIOP protocol being used in the message.
 * This is not equivalent to the IIOP Version although they have the same structures.
 *
 * @since ACT 0.1
 */
public final class Version implements Streamable
{
    /**
     * @since ACT 0.1
     */
    public byte major_;

    /**
     * @since ACT 0.1
     */
    public byte minor_;
 
    /**
     * @since ACT 0.1
     */
    public static final Version Version_1_0 =
		new Version((byte)0x01, (byte)0x00);

    /**
     * Used only in the GIOPMessagefactory
     *
     * @see ORG.omg.CORBA.Streamable
     *
     * @since ACT 0.1
     */
    protected Version() {
    }
    
    /**
     * @param byte major
     * @param byte minor
     *
     * @since ACT 0.1
     */
    public Version(byte major, byte minor) {
		major_ = major;
		minor_ = minor;
    }

    /**
     * Read from the TypeReader stream.
     *
     * @exception java.io.IOException If any IO error occurs
     *
     * @see ORG.omg.CORBA.Streamable#read
     * @see EDU.rutgers.act.presentation.TypeReader
     *
     * @since ACT 0.1
     */
    public void read(TypeReader in) 
		throws java.io.IOException {
		major_ = in.readOctet();
		minor_ = in.readOctet();
    }

    /**
     * Write to the TypeWriter stream.
     * The TypeWriter should have the same transfer syntax as the ts_ member variable.
     *
     * @exception java.io.IOException If any IO error occurs
     *
     * @see ORG.omg.CORBA.Streamable#write
     * @see EDU.rutgers.act.presentation.TypeWriter
     *
     * @since ACT 0.1
     */
    public void write(TypeWriter out)
		throws java.io.IOException {
		out.writeOctet(major_);
		out.writeOctet(minor_);
    }

    /**
     * Compares the version equality
     * 
     * @param Object Version object
     *
     * @return boolean if the versions are the same, return true, if not, return false
     *
     * @since ACT 0.1
     */
    public boolean equals(Object o) {
		if((o != null) && (o instanceof Version)) {
			Version v = (Version)o;
			if(major_ == v.major_ && minor_ == v.minor_) {
				return true;
			}
	    }
	    return false;
    }

    /**
     * Returns a string representation.
     *
     * @return String description
     *
     * @since ACT 0.1
     */
    public String toString() {
        StringBuffer buf = new StringBuffer("GIOP:Version: ").
            append(major_).append(".").append(minor_);
        return String.valueOf(buf);
    }
}
