/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package EDU.rutgers.CORBA.IOP;

import ORG.omg.CORBA.Streamable;
import ORG.omg.CORBA.OctetSequence;
import ORG.omg.CORBA.EMPTYByteArray;

import EDU.rutgers.act.presentation.TypeReader;
import EDU.rutgers.act.presentation.TypeWriter;

/**
 * [CORBA 2.0 Specification 10-19]
 *
 * @since ACT 0.1
 */
public final class ServiceContext implements Streamable
{
    /**
     * @since ACT 0.1
     */
    public int context_id_;

    /**
     * Encapsulation (sequence<octet>)
     *
     * @since ACT 0.1
     */
    public byte[] context_data_ = EMPTYByteArray.EMPTY;

    /**
     * @since ACT 0.1
     */
    public ServiceContext() {
    }

    /**
     * @param int context_id
     * @param byte[] context_data
     *
     * @since ACT 0.1
     */
    public ServiceContext(int context_id, byte[] context_data) {
		context_id_ = context_id;
		context_data_ = context_data;
    }

    /**
     * Read from the TypeReader stream.
     *
     * @exception java.io.IOException If any IO error occurs
     *
     * @see ORG.omg.CORBA.Streamable#read
     * @see EDU.rutgers.act.presentation.TypeReader
     *
     * @since ACT 0.1
     */
    public void read(TypeReader in)
        throws java.io.IOException {
		context_id_ = in.readULong();
		int len = in.readULong();
		context_data_ = new byte[len];
		in.readOctetArray(context_data_, 0, len);
	}    

    /**
     * Write to the TypeWriter stream.
     *
     * @exception java.io.IOException If any IO error occurs
     *
     * @see ORG.omg.CORBA.Streamable#write
     * @see EDU.rutgers.act.presentation.TypeWriter
     *
     * @since ACT 0.1
     */
    public void write(TypeWriter out)
        throws java.io.IOException {
		int len = context_data_.length;
		out.writeULong(context_id_);
		out.writeULong(len);
		out.writeOctetArray(context_data_, 0, len);
	}    

// Object
/*
    public boolean equals(Object o) {
		if((o!= null) && (o instanceof ServiceContext)) {
			ServiceContext sc = (ServiceContext)o;
			String s1 = new String(context_data_);
			String s2 = new String(sc.context_data_);
			if(context_id_ == sc.context_id_ && s1.equals(s2)) {
				return true;
			}
		}
		return false;
    }
*/
    /**
     * Returns a string representation. 
     *
     * @return String description
     *
     * @since ACT 0.1
     */
    public String toString() {
        StringBuffer buf = new StringBuffer("IOP:ServiceContext\n\t").
            append(" context_id_:").append(context_id_).
            append(" context_data ").append(OctetSequence.byteArray2String(context_data_));
        return String.valueOf(buf);
    }
}
