// for emacs only: -*- C++ -*-

// last update: 08/16/97

package EDU.rutgers.CORBA.clntsrv.PortableServer;

import java.net.InetAddress;

import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.OutputStreamWriter;
import java.io.InputStreamReader;

import EDU.rutgers.util.Options;
import ORG.omg.CORBA.*;
import EDU.rutgers.CORBA.IOP.*;
import EDU.rutgers.CORBA.IIOP.*;
import EDU.rutgers.act.presentation.TypeWriter;
import EDU.rutgers.act.presentation.TypeReader;
import EDU.rutgers.CORBA.presentation.*;

import EDU.rutgers.act.protocol.*;
import EDU.rutgers.act.protocol.tcp.*;
import EDU.rutgers.CORBA.GIOP.actions.GIOPClientDataFactory;
import EDU.rutgers.CORBA.GIOP.actions.GIOPServerDataFactory;
import EDU.rutgers.act.protocol.MessageFactory;
import EDU.rutgers.CORBA.GIOP.messages.*;

import EDU.rutgers.act.clntsrv.threadengine.*;


public class ORB extends Thread
{
  // the default port to listen to
  private final short DEFAULT_PORT = 5000;

  // the hexa digit array, needed in stringifying the IOR
  private final byte[] hexaDigits =
    {
      '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
      'a', 'b', 'c', 'd', 'e', 'f'
    };

  public static Options go_;

  // constructors
  protected ORB(String[] args, String[] props)
    {
      started_ = false;
      
      // retain host and port
      host_ = null;
      try {
	host_ = InetAddress.getLocalHost().getHostAddress();
	System.out.println("@ORB.java: local host is <" + host_ + ">");
      }
      catch(java.net.UnknownHostException e)
	{
	  System.out.println("@ORB.java error finding local host: " + e);
	}
      port_ = (short)(DEFAULT_PORT + (System.currentTimeMillis() & 0xFF));
      
      bootstrapHost_ = host_;
      bootstrapPort_ = 900;   // default, specified by CORBA

      setOptions(args);

      //
      // creation of the protocol layer --- client side
      //

      FSMDataFactory clntFactory =
	new GIOPClientDataFactory();
      MessageFactory clntMsgFactory =
	new EDU.rutgers.CORBA.GIOP.messages.GIOPMessageFactory();

      clientSender_ = new ClientSender(clntFactory,
				       clntMsgFactory,
				       new CDRTransferSyntaxFactory());

      // end creation of protocol layer --- client side
      //
      // **********************************************
      //
      // creation of the protocol layer --- server side

      FSMDataFactory srvFactory =
	new GIOPServerDataFactory();
      MessageFactory srvMsgFactory =
	new EDU.rutgers.CORBA.GIOP.messages.GIOPMessageFactory();

      try
	{
	  protSrv_ = new ProtocolServer(srvFactory,
					srvMsgFactory,
					new CDRTransferSyntaxFactory(),
					port_);
	  port_ = protSrv_.getPort();
	}
      catch(java.io.IOException e)
	{
	  System.err.println("error creating ProtocolServer " + e);
	  System.exit(1);
	}


      // end creation of protocol layer --- server side

      // the Thread Engine can not be initialized until POA
      // is initialized, therefore do that in POA_init

    }

  protected void setOptions(String args[])
    {
      // -h bootstrap host for naming service
      // -b bootstrap port for naming service
      // -p local port for the orb

      // this try statement is Bogdan's code ;)
      try
	{
	  go_ = new Options(args, "ORBInitialHost ORBInitialPort ORBPort");
	  String value;
	  if((value = go_.getValueOf("ORBInitialHost")) != "")
	    bootstrapHost_ = value;
	  if((value = go_.getValueOf("ORBInitialPort")) != "")
	    bootstrapPort_ = (short)Integer.parseInt(value);
	  if((value = go_.getValueOf("ORBPort")) != "")
	    port_ = (short)Integer.parseInt(value);
	}
      catch (Exception e)
	{
	  validOptions();
	  System.exit(0);
	}
    }

  protected void validOptions()
    {
      System.out.println("\nunknown option, valid options are:");
      System.out.println("\t-ORBPort <port>\t local port to listen to");
      System.out.println("\t-ORBInitialPort <port>\t bootstrap port for Naming Service (remote)");
      System.out.println("\t-ORBInitialHost <host>\t bootstrap host for Naming Service (remote)");
    }


  // resolve initial references, uses the bootstrap protocols defined
  // by CORBA (at ftp://ftp.omg.org/pub/docs/orbos/97-06-03.pdf)
  public ORG.omg.CORBA.Object resolve_initial_references(String name)
    {
      if(name.equals("NameService"))
	{
	  // create a new IOR for the INIT object reference
	  
	  // create the TypeWriter
	  ByteArrayOutputStream bodyStream = new ByteArrayOutputStream();
	  TypeWriter bodyWriter = new CDRTypeWriter(bodyStream);
	  
	  // create the ProfileBody
	  ProfileBody profileBody = new ProfileBody(bootstrapHost_,
						    bootstrapPort_,
						    "INIT".getBytes());
	  // encapsulate the ProfileBody
	  try
	    {
	      profileBody.write(bodyWriter);
	      bodyWriter.flush();
	    }
	  catch(java.io.IOException ioEx)
	    {
	      System.out.println("Fatal error encapsulating ProfileBody");
	      Runtime.getRuntime().exit(1);
	    }
	  
	  byte[] profileEncapsulation = bodyStream.toByteArray();
	  
	  // put the profile body into the TaggedProfile
	  TaggedProfile taggedProfile =
	    new TaggedProfile(TAG_INTERNET_IOP.value,
			      profileEncapsulation);
	  
	  // Prepare the IOR
	  TaggedProfile[] profiles = new TaggedProfile[1];
	  profiles[0] = taggedProfile;
	  IOR ior = new IOR("", profiles);
	  

          // compose the stub for the InitialReferences interface
	  // by CORBA2.0
	  // available at ftp://ftp.omg.org/pub/docs/orbos/97-06-03.pdf
	  // Slight modifications (non-portable!!!) needed in order
	  // to communicate with JavaIDL's naming service

	  InitialReferences ii = new InitialReferencesStub(ior);
	  
          // String[] services = ii.list();
	  // for(int i = 0; i < services.length; i++)
	  //   System.out.println(services[i]);

	  // JavaIDLFix
	  // this should actually be:
	  return ii.get("NameService");
	  // but JavaIDL has a crooked implementation:
	  // return string_to_object(ii.get("NameService"));

	}

      return (ORG.omg.CORBA.Object)null;
    }

  public void run()
    {
      if(!started_)
	{
	  if(poa_ != null)
	    {
	      started_ = true;
	      try
		{
		  acceptor_.run();
		}
	      catch(java.io.IOException e)
		{
		  System.err.println("error running the acceptor " + e);
		  System.exit(1);
		}
	    }
	}
    }

  public static synchronized ORB init(String[] args, String[] props)
    {
      if(orb_ != null)
	{
	  return orb_;
	}
      return orb_ = new ORB(args, props);
    }

  public static synchronized ORB init()
    {
      return orb_;
    }


  // POA initialization
  public synchronized POA POA_init()
    {
      if(poa_ != null)
	{
	  return poa_;
	}

      // POA creation
      // POA needs to know the host and port for IOR creation
      poa_ = new POA(host_, port_);

      // Thread Engine creation

      // obtaining a Thread-Per-Request RequestHandlerFactory
      RequestHandlerFactory requestHandlerFactory = (RequestHandlerFactory)
 new TPRRequestHandlerFactory((Service)poa_);

      // obtaining a Thread-Per-Connection ConnectionFactory
      ConnectionFactory connectionFactory = (ConnectionFactory)
 new TPCConnectionFactory(requestHandlerFactory);

      // obtaining the Acceptor (Thread Per Request policy)
      acceptor_ = new Acceptor(connectionFactory, protSrv_);

      // end of Thread Engine creation

      // start the ORB thread
      this.start();

      return poa_;
    }

  // string object_to_string(in Object obj)
  public String object_to_string(ORG.omg.CORBA.Object obj)
    {
      ObjectReference o = (ObjectReference)obj;
      IOR ior = o.getIOR();

      // get the TypeWriter
      ByteArrayOutputStream iorStream = new ByteArrayOutputStream();
      TypeWriter iorWriter = new CDRTypeWriter(iorStream);

      // to byte[]
      iorWriter.setByteOrder(true);
      iorWriter.setIndex(0);
      try
 {
   iorWriter.writeBoolean(true);
   /* length!!!
   iorWriter.writeULong(0);
   */
   ior.write(iorWriter);
   iorWriter.flush();
 }
      catch(java.io.IOException ioEx)
 {
   System.out.println("Fatal error stringifying IOR");
   Runtime.getRuntime().exit(1);
 }


      // the stringified IOR
      byte[] iorEncapsulation = iorStream.toByteArray();
      /* length!!!
      int seqLen = iorEncapsulation.length - 5;
      iorEncapsulation[1] = (byte)(seqLen & 0xFF);
      iorEncapsulation[2] = (byte)((seqLen >> 8) & 0xFF);
      iorEncapsulation[3] = (byte)((seqLen >> 16) & 0xFF);
      iorEncapsulation[4] = (byte)((seqLen >> 24) & 0xFF);
      */

      byte[] stringifiedIOR = new byte[4 + 2 * iorEncapsulation.length];
      stringifiedIOR[0] = 'I';
      stringifiedIOR[1] = 'O';
      stringifiedIOR[2] = 'R';
      stringifiedIOR[3] = ':';

      // stringify the IOR
      int i = 0, j = 4;
      for(; i < iorEncapsulation.length; i++)
 {
   stringifiedIOR[j++] = hexaDigits[(iorEncapsulation[i] >> 4) & 0x0F];
   stringifiedIOR[j++] = hexaDigits[iorEncapsulation[i] & 0x0F];
 }

      return new String(stringifiedIOR);
    }

  // read a hexa digit
  protected byte digitValue(byte d)
    {
      if((d >= 48) && (d < 58))
 return (byte)(d - 48);
      if((d >= (byte)'a') && (d <= (byte)'z'))
 return (byte)(d - (byte)'a' + 10);
      if((d >= (byte)'A') && (d <= (byte)'Z'))
 return (byte)(d - (byte)'A' + 10);
      // error
      return -1;
    }


  // Object string_to_object(in string str)
  public ORG.omg.CORBA.Object string_to_object(String str)
    {
      byte[] stringifiedIOR = str.getBytes();
      if(stringifiedIOR.length < 4)
 {
   // illegal IOR
   return null;
 }

      if((stringifiedIOR[0] != 'I') ||
  (stringifiedIOR[1] != 'O') ||
  (stringifiedIOR[2] != 'R') ||
  (stringifiedIOR[3] != ':'))
 {
   // illegal IOR
   return null;
 }

      byte[] iorEncapsulation = new byte[(stringifiedIOR.length - 4) >> 1];

      int i = 4, j = 0;
      // unstringify the IOR
      for(; i < stringifiedIOR.length; i += 2)
 {
   iorEncapsulation[j++] =
     (byte)((digitValue(stringifiedIOR[i]) << 4) +
     digitValue(stringifiedIOR[i + 1]));

     // Byte.parseByte(new String(stringifiedIOR, i, 2), 16);
 }

      // get the IOR from the IOR encapsulation
      ByteArrayInputStream stream = new ByteArrayInputStream(iorEncapsulation);
      TypeReader in = new CDRTypeReader(stream);

      IOR ior;
      in.setIndex(0);
      try
 {
   in.setByteOrder(in.readBoolean());
   /* length!!!
   int seqLen = in.readULong();
   */
   ior = new IOR();
   ior.read(in);
 }
      catch (java.io.IOException e)
 {
   // some problems, just return null
   return null;
        }

      return (ORG.omg.CORBA.Object) new ObjectReference(ior);
    }

  // specific IDL/Java mapping
  public void connect(ORG.omg.CORBA.Object o)
    {
      poa_.activate_object((Skeleton)o);
    }

  public void disconnect(ORG.omg.CORBA.Object o)
    {
      // to be implemented!!!
    }

  public ClientSender getClientSender()
    {
      return clientSender_;
    }

  // private data members
  protected static ORB orb_ = null;
  protected static POA poa_ = null;

  protected boolean started_;
  protected ClientSender clientSender_;
  protected Acceptor acceptor_;
  protected ProtocolServer protSrv_;

  protected String host_;
  protected short port_;

  protected String bootstrapHost_;
  protected short  bootstrapPort_;
}


