// for emacs only: -*- C++ -*-

// last update: 08/22/97

package EDU.rutgers.CORBA.clntsrv.PortableServer;

import EDU.rutgers.act.protocol.*;
import EDU.rutgers.act.protocol.tcp.*;
import EDU.rutgers.CORBA.GIOP.messages.*;

import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.OutputStreamWriter;
import java.io.InputStreamReader;
import EDU.rutgers.act.presentation.TypeWriter;
import EDU.rutgers.act.presentation.TypeReader;
import ORG.omg.CORBA.Principal;
import EDU.rutgers.CORBA.IOP.IOR;
import EDU.rutgers.CORBA.IOP .ServiceContext;
import EDU.rutgers.CORBA.presentation.*;
import EDU.rutgers.CORBA.IIOP.ProfileBody;
import EDU.rutgers.act.endpoint.Endpoint;

public class ObjectReference implements ORG.omg.CORBA.Object 
{
  // constructors
  public ObjectReference()
    {
      ior_ = null;
    }

  public ObjectReference(IOR ior)
    {
      ior_ = ior;
    }
  

  // public methods

  // checks object type at runtime
  public boolean _is_a(String Identifier)
    {
      // this should be improved in the future to make narrow
      // calls to the implementation...
      // but for now it just checks the IOR type
      // return ior_.type_id_.compareTo(Identifier) == 0;
      return true;
    }
  
  public boolean _is_equivalent(ORG.omg.CORBA.Object that)
    {
      // for the moment two Objects are equivalent only if they
      // are equal...
      return that._is_a(ior_.type_id_);
    }
  
  public boolean _non_existent()
    {
      // For the moment the object is always existent, since
      // this is it's Object Reference. But in the future
      // remote call checks should be done
      return false;
    }
  
  public int _hash(int maximum)
    {
      // not implemented yet
      return 0;
    }
  
  public ORG.omg.CORBA.Object _duplicate()
    {
      // not implemented yet
      return this;
    }
  
  public void _release()
    {
      // not implemented yet
    }

  public boolean equals(Object o)
    {
      if(o instanceof ORG.omg.CORBA.Object)
	{
	  ORB orb = ORB.init();
	  String s1 = orb.object_to_string(this),
	         s2 = orb.object_to_string((ORG.omg.CORBA.Object)o);
	  return s1.equals(s2);
	}
      return false;
    }
  

  // these are mainly for DII, not implemented yet

  // ImplementationDef _get_implementation();
  // InterfaceDef _get_interface();
  // Request _request(String s);
  // Request _create_request(Context ctx,
  //			  String operation,
  //			  NVList arg_list,
  //			  NamedValue result);
  // Request _create_request(Context ctx,
  //			  String operation,
  //			  NVList arg_list,
  //			  NamedValue result,
  //                      ExceptionList exclist,
  //                      ContextList ctxlist);


  // internal methods, used only by ACT
  protected TypeWriter getTypeWriter(String operation)
    {
      profile_ = ior_.getIIOPProfile();
      byte[] objectKey = profile_.object_key_;


      //
      // Compose the Request message
      //

      byte[] context_data = "".getBytes();
      ServiceContext[] service_context = new ServiceContext[0 /* 1 !!! */];

      // I replaced the above '1' to send no service context
      // for testing purposes -> to see if this is the problem
      // in communicating with JavaIDL Name Server using bootstrapping

      for (int i = 0; i < service_context.length; i ++)
	{
	  service_context[i] = new ServiceContext(1, context_data);
        }

      // !!!!
      int requestId = getNextRequestId(); 
      boolean response_expected = true;
      Principal requesting_principal = new Principal("".getBytes());
      request_ =
	new GIOPRequestMessage(service_context, 
			       requestId, 
			       response_expected, 
			       objectKey, 
			       operation, 
			       requesting_principal);

      bodyStream_ = new ByteArrayOutputStream();
      outWriter_ = new CDRTypeWriter(bodyStream_);
      outWriter_.setByteOrder(request_.byte_order_); // inherited

      // aligned with the GIOP header
      outWriter_.setIndex(request_.getBeforeBodySize()); 

      return outWriter_;
    }
  

  protected TypeReader getTypeReader()
    {
      try
	{
	  outWriter_.flush();

	  // write to the body
	  byte[] bodyBytes;

	  bodyBytes = bodyStream_.toByteArray();
	  request_.setRequestBody(bodyBytes);
        }
      catch (java.io.IOException e)
	{
	  System.err.println("Error composing Request Message");
	  System.exit(1);
        }

      // Request message created

      request_.setEndpoint(new Endpoint(profile_.host_, 
					0x0000FFFF & profile_.port_));

      // send the message and get the result!
      GIOPReplyMessage reply = (GIOPReplyMessage)invoke(request_);
      
      // get the argument (the string)
      ByteArrayInputStream stream = 
	new ByteArrayInputStream(reply.getReplyBody());
      TypeReader in = new CDRTypeReader(stream);

      in.setIndex(reply.getBeforeBodySize());
      in.setByteOrder(reply.byte_order_);

      return in;
    }
  
  protected Message invoke(Message request)
    {
      ORB.init().getClientSender().sendMessage((Message)request);
      ClientReceiver clientReceiver =
	new ClientReceiver(request.getTransactionId());

      Message reply = null;
      try
	{
	  reply = clientReceiver.receiveMessage();
	}
      catch(java.io.IOException ioEx)
	{
	  // fatal error
	  System.out.println("Error receiving reply data");
	  System.exit(1);
	}
      return reply;
    }

  public IOR getIOR()
    {
      return ior_;
    }

  synchronized static int getNextRequestId()
    {
      return currentRequestId_++;
    }

  public void setIOR(IOR ior)
    {
      ior_ = ior;
    }

  // data members
  protected IOR ior_;
  static protected int currentRequestId_ = 0;

  protected GIOPRequestMessage request_;
  protected TypeWriter outWriter_;
  protected ByteArrayOutputStream bodyStream_;
  protected ProfileBody profile_;

}

