// for emacs only: -*- C++ -*-

// last update: 08/13/97

package EDU.rutgers.CORBA.clntsrv.PortableServer;

import EDU.rutgers.act.protocol.*;
import EDU.rutgers.CORBA.GIOP.messages.*;

import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.OutputStreamWriter;
import java.io.InputStreamReader;
import EDU.rutgers.CORBA.IOP.IOR;
import EDU.rutgers.act.presentation.TypeWriter;
import EDU.rutgers.act.presentation.TypeReader;
import EDU.rutgers.CORBA.presentation.CDRTypeReader;
import EDU.rutgers.CORBA.presentation.CDRTypeWriter;

public abstract class Skeleton extends ObjectReference implements Servant
{
  public Skeleton()
    {
      objType_ = "ObjectReference";
    }

  //-------------------------------------------------------
  // here starts the code that should be implemented by the
  // IDL compiler - ImplBase class
  //-------------------------------------------------------
    
  protected abstract void callMethod(String operation,
				     TypeReader in,
				     TypeWriter out);
  //-------------------------------------------------------
  // here ends the code that should be implemented by the
  // IDL compiler
  //-------------------------------------------------------

  public void handleRequest(EDU.rutgers.act.protocol.Message m, 
			    EDU.rutgers.act.protocol.Sender s)
  {
    // just print something for testing purposes
    GIOPRequestMessage request = (GIOPRequestMessage)m;

    // System.out.println("TestSkeleton.handleRequest objectId is <"
    //		       +stringify(request.object_key_)
    //		       +">");
    // System.out.println("Message is: \n"+request);

    // open input stream (from the Request Body)
    ByteArrayInputStream inStream = 
      new ByteArrayInputStream(request.getRequestBody());
    CDRTypeReader in = new CDRTypeReader(inStream);

    in.setIndex(request.getBeforeBodySize());
    in.setByteOrder(request.byte_order_);

    // compose the reply and open the output stream (Reply Body)
    GIOPReplyMessage replyMsg = 
      new GIOPReplyMessage(request.service_context_, 
			   request.request_id_,
			   ReplyStatusType.NO_EXCEPTION);

    ByteArrayOutputStream outStream = new ByteArrayOutputStream();
    CDRTypeWriter out = new CDRTypeWriter(outStream);
    // to byte[]
    out.setByteOrder(replyMsg.byte_order_);
    out.setIndex(replyMsg.getBeforeBodySize());

    // call the IDL-generated callMethod
    callMethod(request.operation_, in, out);

    byte[] replyBody = outStream.toByteArray();
	
    // set the reply body
    replyMsg.setReplyBody(replyBody);
            
    // dump the reply message
    // now it's the moment to send the message over the network
    // System.out.println("Reply message is:");
    // System.out.println("" + replyMsg);

    try
      {
	s.sendMessage(replyMsg);
      }
    catch(java.io.IOException e)
      {
	System.err.println("error sending reply message: "+e);
      }
  }

  // garbage code, testing purposes
  public String stringify(byte[] bytearray)
    {
      int i, j;
      byte[] aux = new byte[bytearray.length<<1];

      byte[] digit = "0123456789ABCDEF".getBytes();

      for(i = 0; i < bytearray.length; i++)
	{
	  j = i << 1;
	  aux[j] = digit[bytearray[i] >> 4];
	  aux[j+1] = digit[bytearray[i] & (byte)0x0F];
	}

      return new String(aux);
    }

  // data members
  public String objType_;
}

