// for emacs: -*- C++ -*-

// last update: 07/28/97

package EDU.rutgers.act.clntsrv.threadengine;

import java.io.*;

import EDU.rutgers.act.protocol.*;

/**********************************************
 * Connection                                 *
 **********************************************/
/**********************************************
 * This class manages a connection.  It runs
 * in a loop, reading the next message from
 * a ProtocolReceiver, it uses the
 * RequestHandlerFactory to create a
 * RequestHandler, it calls that RequestHandler
 * and loops again. The loop is implemented
 * by the method "run". The loop ends when 
 * the client closes the connection.
 **********************************************/
class Connection implements Runnable { 
  // constructors
  public Connection(RequestHandlerFactory rhf, Protocol protocol)
    {
      requestHandlerFactory_ = rhf;
      protocol_ = protocol;
    }

  // public methods
  public synchronized void run()
    {
      Message request = null;
      try 
	{
	  request = protocol_.getServerReceiver().receiveMessage();
	}
      catch(ORG.omg.CORBA.MARSHAL ex1)
	{
	  request = null;
	}
      catch(java.io.IOException ex2)
	{
	}
      
      while(request != null)
	{
	  try
	    {
	      requestHandlerFactory_.createRequestHandler(
				     request,
				     protocol_.getServerSender());
	      request = protocol_.getServerReceiver().receiveMessage();
	    }
      catch(ORG.omg.CORBA.MARSHAL ex1)
	{
	  request = null;
	}
	  catch(java.io.IOException exception)
	    {
	    }
	}
      // Protocol object cannot be destroyed here because there can
      // be several requestHandler objects keeping pointers to it.
      // When should we destroy it, then? A solution can be the var
      // pointers, incrementing an internal counter every time the
      // handle is duplicated.  Every requestHandler calls a method
      // let's say "dispose" on Protocol and the last one turns off
      // the light.
    }
  

  // data members
  RequestHandlerFactory requestHandlerFactory_;
  Protocol protocol_;
}
