/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package EDU.rutgers.act.endpoint;

import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * Endpoint contains information for creating an endpoint
 * object.
 */
public final class Endpoint
{
  public String hostName_;
  public int port_;
  protected String key_;
  
  public Endpoint(String hostName, int port)
    {
      hostName_ = hostName; 
      port_ = port; 
      try 
	{
	  key_ = toKey(InetAddress.getByName(hostName), port);
	}
      catch(UnknownHostException e)
	{
	  key_ = "";
	  System.out.println("Endpoint.Endpoint(): " + e);
	}
    }

  public boolean equals(Object o)
    {
      if(o != null && o instanceof Endpoint)
	{
	  Endpoint ep = (Endpoint)o;
	  if(this == ep)
	    return true;
	  if(this.key_.equals(ep.key_))
	    return true; 
        }
      return false; 
    }

  public int hashCode()
    {
      return key_.hashCode();
    }

  public String toString()
    {
      return "IIOP:Endpoint\n\t" + 
	"hostName: " + hostName_ + ":" + "port: " + port_; 
    }

  // get the unique key for the IP and port
  String toKey(InetAddress ia, int port)
    {
      byte[] p = new byte[6];
      System.arraycopy(ia.getAddress(), 0, p, 0, 4);
      
      p[4] = (byte)port;
      p[5] = (byte)(port >> 8);
      
      return new String(p);
    }
}

