/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package EDU.rutgers.act.presentation;

import java.io.OutputStream;
import java.io.FilterWriter;
import java.io.Writer;

import ORG.omg.CORBA.BAD_PARAM;
import ORG.omg.CORBA.DATA_CONVERSION;

/**
 * Write primitives and primitive arrays from stream based on sepcific transfer syntax.
 *
 * @see TransferSyntax
 * @see TypeReader
 *
 *
 * @since ACT 0.1
 */
public abstract class TypeWriter 
//	extends FilterWriter
{
    /**
     * @param Writer the actual reader used to perform writing
     *
     * @since ACT 0.1
     */
    protected OutputStream out_;
    protected TypeWriter(OutputStream out) {
//		super(out);
		out_ = out;
		index_ = 0;
    }

    /**
     * Index to be aligned with
     *
     * @since ACT 0.1
     */
	protected int index_;

    /**
     * Byte order : true for little endian, false for big endian
     *
     * @since ACT 0.1
     */
	protected boolean littleEndian_;

    /**
     * @param int align index
     *
     * @see #index_
     *
     * @since ACT 0.1
     */
	public void setIndex(int index) { 
		index_ = index; 
	}

    /**
     * @return int align index
     *
     * @see #index_
     *
     * @since ACT 0.1
     */
	public int getIndex() { 
		return index_; 
	}

    /**
     * @param boolean true for little endian, false for big endian
     *
     * @see littleEndian
     *
     * @since ACT 0.1
     */
	public void setByteOrder(boolean littleEndian) {
		littleEndian_ = littleEndian;
	}

    /**
     * @return boolean true for little endian, false for big endian
     *
     * @see littleEndian
     *
     * @since ACT 0.1
     */
    public boolean getByteOrder() {
        return littleEndian_;
    }


	public void flush()
		throws java.io.IOException
	{
 		out_.flush();
	}

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract void writeBoolean(boolean b)
		throws java.io.IOException;    

    /**
     * @exception java.io.IOException if any IO error occurs
     * @exception DATA_CONVERSION if the char value greater than 0xff
     *
     * @since ACT 0.1
     */
	public abstract void writeChar(char c)
		throws java.io.IOException, DATA_CONVERSION;    

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
	public abstract void writeOctet(byte b)
		throws java.io.IOException;    

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
	public abstract void writeShort(short s)
		throws java.io.IOException;
    
    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract void writeUShort(short us)
		throws java.io.IOException;    

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
	public abstract void writeLong(int l)
		throws java.io.IOException;

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract void writeULong(int ul)
		throws java.io.IOException;    

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
	public abstract void writeLongLong(long ll)
		throws java.io.IOException;

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract void writeULongLong(long ull)
		throws java.io.IOException;    

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
	public abstract void writeFloat(float f)
		throws java.io.IOException;

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
    public abstract void writeDouble(double d)
		throws java.io.IOException;    

    /**
     * @exception java.io.IOException if any IO error occurs
     *
     * @since ACT 0.1
     */
	public abstract void writeString(String str)
		throws java.io.IOException;

    /**
     * @param boolean[] boolean array
     * @param int offset of the array to begin to write out
     * @param int number of boolean to write out
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void writeBooleanArray(boolean[] ba, int offset, int len)
		throws java.io.IOException, BAD_PARAM {       
        if (ba == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index = offset; index < (offset + len); index++) {
            if (index >= ba.length) {
                break;
            }
            writeBoolean(ba[index]); 
        }
	}

    /**
     * @param char[] char array (no UNICODE)
     * @param int offset of the array to begin to write out
     * @param int number of char to write out
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void writeCharArray(char[] ca, int offset, int len)
		throws java.io.IOException, BAD_PARAM
	{
        if (ca == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index = offset; index < (offset + len); index++) {
            if (index >= ca.length) {
                break;
            }
            writeChar(ca[index]); 
        }
	}

    /**
     * @param byte[] byte array (convert to unsigned)
     * @param int offset of the array to begin to write out
     * @param int number of byte to write out
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void writeOctetArray(byte[] oa, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (oa == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index = offset; index < (offset + len); index++) {
            if (index >= oa.length) {
                break;
            }
            writeOctet(oa[index]); 
        }
	}

    /**
     * @param short[] short array 
     * @param int offset of the array to begin to write out
     * @param int number of short to write out
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void writeShortArray(short[] sa, int offset, int len)
        throws java.io.IOException, BAD_PARAM {
        if (sa == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index = offset; index < (offset + len); index++) {
            if (index >= sa.length) {
                break;
            }
            writeShort(sa[index]); 
        }
	}

    /**
     * @param short[] short array
     * @param int offset of the array to begin to write out
     * @param int number of short to write out
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void writeUshortArray(short[] usa, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (usa == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index = offset; index < (offset + len); index++) {
            if (index >= usa.length) {
                break;
            }
            writeUShort(usa[index]); 
        }
	}

    /**
     * @param int[] int array (no UNICODE)
     * @param int offset of the array to begin to write out
     * @param int number of int to write out
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void writeLongArray(int[] la, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (la == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index = offset; index < (offset + len); index++) {
            if (index >= la.length) {
                break;
            }
            writeLong(la[index]); 
        }
	}

    /**
     * @param int[] int array (no UNICODE)
     * @param int offset of the array to begin to write out
     * @param int number of int to write out
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void writeULongArray(int[] ula, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (ula == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index = offset; index < (offset + len); index++) {
            if (index >= ula.length) {
                break;
            }
            writeULong(ula[index]); 
        }
	}

    /**
     * @param long[] long array (no UNICODE)
     * @param int offset of the array to begin to write out
     * @param int number of long to write out
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void writeLongLongArray(long[] lla, int offset, int len)
        throws java.io.IOException, BAD_PARAM {
        if (lla == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index = offset; index < (offset + len); index++) {
            if (index >= lla.length) {
                break;
            }
            writeLongLong(lla[index]); 
        }
	}

    /**
     * @param long[] long array (no UNICODE)
     * @param int offset of the array to begin to write out
     * @param int number of long to write out
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void writeULongLongArray(long[] ulla, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (ulla == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index = offset; index < (offset + len); index++) {
            if (index >= ulla.length) {
                break;
            }
            writeULongLong(ulla[index]); 
        }
	}

    /**
     * @param float[] float array (no UNICODE)
     * @param int offset of the array to begin to write out
     * @param int number of float to write out
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void writeFloatArray(float[] fa, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (fa == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index = offset; index < (offset + len); index++) {
            if (index >= fa.length) {
                break;
            }
            writeFloat(fa[index]); 
        }
	}

    /**
     * @param double[] double array (no UNICODE)
     * @param int offset of the array to begin to write out
     * @param int number of double to write out
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void writeDoubleArray(double[] da, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (da == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index = offset; index < (offset + len); index++) {
            if (index >= da.length) {
                break;
            }
            writeDouble(da[index]); 
        }
	}

    /**
     * @param String[] String array (no UNICODE)
     * @param int offset of the array to begin to write out
     * @param int number of String to write out
     *
     * @exception java.io.IOException if any IO error occurs
     * @exception BAD_PARAM if array is null
     *
     * @since ACT 0.1
     */
    public void writeStringArray(String[] stra, int offset, int len)
		throws java.io.IOException, BAD_PARAM {
        if (stra == null) {
            throw new BAD_PARAM(); 
        }
        int index;
        for (index = offset; index < (offset + len); index++) {
            if (index >= stra.length) {
                break;
            }
            writeString(stra[index]); 
        }
	}
}
