// for emacs only: -*- C++ -*-

// Aug. 14, 1997
// Modified by Weicong Wang

/*
   --------------------------------------------------------------
   FILE: FSMAction.java
   --------------------------------------------------------------
   AUTHOR:		Cristian Francu
   LAST UPDATE:		Fri Jul 11 12:01:37 EDT 1997
   --------------------------------------------------------------
*/
/*
--------------------------------------------------------------
DESCRIPTION:

Interface of the Finite State Machine Action object. This object
describes the interface for the object residing in the state table
of the Finite State Machine object, used to code protocols.
--------------------------------------------------------------
*/

package EDU.rutgers.act.protocol;

public interface FSMAction
{
    // constants
    // states are integers starting from 0
    public final int START = 0;
    public final int EXIT  = 1;
    public final int ERROR = 2;

    // - receives:
    //    - 'current' the message that generated this transition
    //    - 'previous' the last sent message
    // - performs some computations in order to
    //    - decide the following state
    //    - decide the message to be transmitted
    // - returns the following state for the FSM and the message
    //     to be transmitted if any, in 'next'
    public int run(Message previous, Message current, Message[] next);
}

