// for emacs only: -*- C++ -*-

// Aug. 14, 1997
// Modified by Weicong Wang

/*
   --------------------------------------------------------------
   FILE: FSMData.java
   --------------------------------------------------------------
   AUTHOR:		Cristian Francu
   LAST UPDATE:		Fri Jul 11 16:59:33 EDT 1997
   --------------------------------------------------------------
*/
/*
--------------------------------------------------------------
DESCRIPTION:

Implementation file for the Finite State Machine Data object. This
object keeps the data needed by the finite state machine
engine in order to define a specific fsm for a specific
protocol. FSMData object contains a matrix of actions, an
array recording which states are final, etc.
--------------------------------------------------------------
*/

package EDU.rutgers.act.protocol;

public class FSMData
{
  // Data members
  protected FSMAction[][] fsmTable_; // the table of pointers to the actions
  protected boolean[] isFinal_;// array of booleans, true if the state is final
  protected int state_;        // current state of the FSM

  protected Message[] next = { null }; // pointer to a message handler

  // constructors
  public FSMData(FSMAction[][] fsmTable,
		 boolean[] isFinal,
		 int initialState)
    {
      fsmTable_ = fsmTable;
      isFinal_ = isFinal;
      state_ = initialState;
    }
  
  // methods
  public boolean isFinal() { return isFinal_[state_]; }

  public Message advance(Message previous, Message current)
    {
      if(state_ == FSMAction.EXIT)    // restart if EXIT is reached
	{
	  // System.out.println("FSMData reached EXIT and goes to start!");
	  state_ = FSMAction.START;
	}

      FSMAction action = fsmTable_[state_][current.getType()];
      state_ = action.run(previous, current, next);
      return next[0];
    }

  public void finalize()
    {
      fsmTable_ = null;
      isFinal_ = null;
    }
  
}
