/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1998 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

// last modification 01/20/98
// 03/12/98 added verbose mode

package EDU.rutgers.act.protocol.mcast;

import java.io.*;

import EDU.rutgers.util.Verbose;
import EDU.rutgers.act.endpoint.*;
import EDU.rutgers.act.endpoint.mcast.*;
import EDU.rutgers.act.protocol.*;
import EDU.rutgers.act.presentation.TransferSyntaxFactory;
import EDU.rutgers.act.presentation.TypeWriter;

/**
   A very simple implementation of the Sender interface, client side.
   This implementation cannot deal properly with multiple messages
   sent under the same Transaction Id (even though it works in
   situations where sends and receives alternate).
  
   @see Sender
  
   @since ACT 0.1
 */
public class ClientSender implements Sender
{
  // data members
  protected Lsocket socket_;
  protected TypeWriter out_;
  protected int ttl_;
  protected Endpoint ep_;
  protected TransferSyntaxFactory tsf_;
  /**
     
     messages exchange, all of them wearing the same transaction id.
    
     @since ACT 0.1
   */
  public ClientSender(TransferSyntaxFactory tsf,
		      int ttl)
    {
      ttl_ = ttl;
      tsf_ = tsf;
    }

  public void setEndpoint(Endpoint ep)
    {
      ep_ = ep;
      socket_ = new Lsocket(ep.hostName_, ep.port_, ttl_);
      out_ = tsf_.createTypeWriter(
	       new BufferedOutputStream(socket_.getOutputStream()));
    }

  public synchronized void sendMessage(Message message)
    {
      Endpoint ep = message.getEndpoint();
      if(ep != null)
	{
	  if(!ep_.equals(ep))
	    {
	      // shut down
	      socket_.close();
	      setEndpoint(ep);
	    }
	  
	  try
	    {
	      message.write(out_);
	      out_.flush();
	    }
	  catch(IOException e)
	    {
	      System.out.println("@ClientSender: " + e);
	    }
	}
      else
	{
	  System.out.println("@ClientSender: null destination");
	}
    }

  InputStream accept()
    {
      return socket_.accept();
    }
}
