/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1998 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

// last modification 01/20/98
// 03/12/98 added verbose mode

package EDU.rutgers.act.protocol.tcp;

import EDU.rutgers.act.protocol.*;
import EDU.rutgers.util.Verbose;
import EDU.rutgers.act.presentation.TransferSyntaxFactory;

/**
   A very simple implementation of the Sender interface, client side.
   This implementation cannot deal properly with multiple messages
   sent under the same Transaction Id (even though it works in
   situations where sends and receives alternate).
  
   @see Sender
  
   @since ACT 0.1
 */
public class ClientSender implements Sender
{
  // data members
  protected FSMDataFactory fsmDataFactory_;
  protected MessageFactory messageFactory_;
  protected TransferSyntaxFactory tsf_;
  protected SocketHandler sh_;
  static java.util.Hashtable receivers_;

  /**
     
     messages exchange, all of them wearing the same transaction id.
    
     @since ACT 0.1
   */
  public ClientSender(FSMDataFactory fsmDataFactory,
		      MessageFactory messageFactory,
		      TransferSyntaxFactory dataTsf) {
    fsmDataFactory_ = fsmDataFactory;
    messageFactory_ = messageFactory;
    tsf_ = dataTsf;
            if(Verbose.verbose)
	      {
		Verbose.print("ClientSender",
			      "ClientSender(3)",
			      "creating the receivers hash table");
	      }
    receivers_ = new java.util.Hashtable();
            if(Verbose.verbose)
	      {
		Verbose.print("ClientSender",
			      "ClientSender(3)",
			      "hash table created: " + receivers_);
	      }
            if(Verbose.verbose)
	      {
		Verbose.print("ClientSender",
			      "ClientSender(3)",
			      "creating the SocketHandler object with arguments: " +
			      messageFactory_ + " and " +
			      tsf_ + " and " + "null");
	      }
    sh_ = new SocketHandler(messageFactory_, tsf_, null);
            if(Verbose.verbose)
	      {
		Verbose.print("ClientSender",
			      "ClientSender(3)",
			      "SocketHandler created: " + sh_);
	      }
  }

  public synchronized void sendMessage(Message message) {

            if(Verbose.verbose)
	      {
		Verbose.print("ClientSender",
			      "sendMessage",
			      "message to be sent is: " + message);
	      }
    String key = Integer.toString(message.getTransactionId(), 16);
            if(Verbose.verbose)
	      {
		Verbose.print("ClientSender",
			      "sendMessage",
			      "key is: " + key);
	      }
    ClientCommunicator cm = (ClientCommunicator)receivers_.get(key);
            if(Verbose.verbose)
	      { 
		Verbose.print("ClientSender",
			      "sendMessage",
			      "the ClientCommunicator for this key is: " + cm);
	      }
    if(cm != null) // there an FSM already created for
                               // this transaction
      {
	        if(Verbose.verbose)
		  {
		    Verbose.print("ClientSender",
				  "sendMessage",
				  "cm not null");
		  }
	cm.setMessage(message);
      }
    else
      {
    	        if(Verbose.verbose)
		  {
		    Verbose.print("ClientSender",
				  "sendMessage",
				  "creating FSMData");
		  }
	FSMData fsmData = fsmDataFactory_.createFSMData();
	        if(Verbose.verbose)
		  {
		    Verbose.print("ClientSender",
				  "sendMessage",
				  "FSMData created: " + fsmData);
		  }
	        if(Verbose.verbose)
		  {
		    Verbose.print("ClientSender",
				  "sendMessage",
				  "creating FSM");
		  }
	FSM fsm = new FSM(fsmData, sh_);
	        if(Verbose.verbose)
		  { 
		    Verbose.print("ClientSender",
				  "sendMessage",
				  "FSM created: " + fsm);
		  }
		if(Verbose.verbose)
		  {
		    Verbose.print("ClientSender",
				  "sendMessage",
				  "creating ClientCommunicator");
		  }
	cm = new ClientCommunicator(fsm, message);
		if(Verbose.verbose)
		  {
		    Verbose.print("ClientSender",
				  "sendMessage",
				  "ClientCommunicator created: " + cm);
		  }
		if(Verbose.verbose)
		  {
		    Verbose.print("ClientSender",
				  "sendMessage",
				  "adding the ClntComm to the hash table");
		  }
	receivers_.put(key, cm);
		if(Verbose.verbose)
		  {
		    Verbose.print("ClientSender",
				  "sendMessage",
				  "ClntComm added to the hash table");
		  }
      }
  }
}
