/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1998 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

// last modification 01/20/98
// Aug. 14, 1997
// Modified by Weicong Wang
// 03/29/98 added verbose mode

/*
--------------------------------------------------------------
DESCRIPTION:

Implementation file for the Finite State Machine object. This
object implements the engine for  a generic state machine, used
to code protocols.
Right now it is used (or going to be used) to implement IIOP
protocol. It uses the state matrix implementation model. Yup,
not too easy to modify, not too easy to read, but very
efficient. It tries to be general...
--------------------------------------------------------------
*/

package EDU.rutgers.act.protocol.tcp;

import EDU.rutgers.util.Verbose;
import EDU.rutgers.act.protocol.*;

public class FSM
{
    protected FSMData fsmData_;        // handle to specific state data
    protected SocketHandler messageH_; // object for exchanging messages
    protected int fsmId_;              // the id used when no tid is provided


    public FSM(FSMData fsmData, SocketHandler messageH) {
        fsmData_ = fsmData;
        messageH_ = messageH;
    }

    public FSM(FSMData fsmData, SocketHandler messageH, int fsmId) {
        fsmData_ = fsmData;
        messageH_ = messageH;
	fsmId_ = fsmId;
    }

    // activates the automaton, given the
    // input message. It returns the output message
    public Message run(Message msgIn) 
        throws java.io.IOException {
        Message previous = msgIn;
        Message current = null;
        Message next = null;
	boolean nothingToBeTransmitted = msgIn == null;

              if(Verbose.verbose)
		{
		  Verbose.print("FSM",
				"run",
				"nothingToBeTransmitted=" +
				nothingToBeTransmitted);
		}
        // Is there something to be transmitted in the beginning?
        if(!nothingToBeTransmitted) {
	        // - select the right action from the table
	        // - launch the action
	        next = fsmData_.advance(null, msgIn);

		              if(Verbose.verbose)
				{
				  Verbose.print("FSM",
						"run",
						"sending message " +
						next);
				}
	        // send the FSM response, if any
	        messageH_.sendMessage(next, fsmId_);
		              if(Verbose.verbose)
				{
				  Verbose.print("FSM",
						"run",
						"message sent successfully");
				}
	    }

		          if(Verbose.verbose)
			    {
			      Verbose.print("FSM",
					    "run",
					    "final state = " + 
					    fsmData_.isFinal());
			    }
		          if(Verbose.verbose)
			    {
			      Verbose.print("FSM",
					    "run",
					    "nothingToBeTransmitted = " + 
					    nothingToBeTransmitted);
			    }
        while(!fsmData_.isFinal() // not a final state -> loop again
              || nothingToBeTransmitted) // ... or it's the first loop (EXIT -> START transition) -> loop 
	  {
	    // we entered the loop once, clear the status of first time
	    nothingToBeTransmitted = false;

		          if(Verbose.verbose)
			    {
			      Verbose.print("FSM",
					    "run",
					    "receiving message");
			    }
            // block for a message
	    if((current = 
                    messageH_.recvMessage(
                          (next == null)?fsmId_:next.getTransactionId()
                    )
               ) != null)
	      {
		          if(Verbose.verbose)
			    {
			      Verbose.print("FSM",
					    "run",
					    "received message: " +
					    current);
			    }
		
		// launch the action
		next = fsmData_.advance(previous, current);
      
		          if(Verbose.verbose)
			    {
			      Verbose.print("FSM",
					    "run",
					    "message to be sent: " +
					    next);
			    }
		
		// send the FSM response, if any
		if(next != null) {
		          if(Verbose.verbose)
			    {
			      Verbose.print("FSM",
					    "run",
					    "sending message");
			    }
		
		  messageH_.sendMessage(next, fsmId_);
		}

		// set the last transmitted message
		previous = next;
	      }
	    else
	      {
		// System.out.println("FSM.java: null message!");
		return null;
	      }
		          if(Verbose.verbose)
			    {
			      Verbose.print("FSM",
					    "run",
					    "final state = " + 
					    fsmData_.isFinal());
			    }
		          if(Verbose.verbose)
			    {
			      Verbose.print("FSM",
					    "run",
					    "nothingToBeTransmitted = " + 
					    nothingToBeTransmitted);
			    }
	  }
        return current;
    }

    public void finalize() 
        throws java.io.IOException {
        fsmData_.finalize();
        fsmData_ = null;
        messageH_ = null;
    }
}

