/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

// last modification: 01/23/98
// 03/12/98 added verbose mode

package EDU.rutgers.act.protocol.tcp;

import EDU.rutgers.util.Verbose;

import java.net.*;
import java.io.*;
import java.util.Hashtable;

import EDU.rutgers.act.presentation.TypeReader;
import EDU.rutgers.act.presentation.TransferSyntaxFactory;
import EDU.rutgers.act.protocol.*;

class SocketReader extends Thread
{
  protected SocketInfo socket_;
  SocketHandler sh_;
  protected Hashtable fsms_;
  TransferSyntaxFactory tsf_;
  MessageFactory factory_;
  InputStream reader_;
  int fsmId_;

  public SocketReader(SocketInfo socket,
		      SocketHandler sh,
		      Hashtable fsms,
		      TransferSyntaxFactory tsf,
		      MessageFactory factory,
		      int fsmId)
    {
      socket_ = socket;
      sh_ = sh;
      fsms_ = fsms;
      tsf_ = tsf;
      factory_ = factory;
      fsmId_ = fsmId;

              if(Verbose.verbose)
		{
		  Verbose.print("SocketReader",
				"SocketReader",
				"entering constructor");
		}
      try
	{
	  reader_ = new BufferedInputStream(socket_.socket_.getInputStream());
	}
      catch(java.io.IOException e)
	{
              if(Verbose.verbose)
		{
		  Verbose.print("SocketReader",
				"SocketReader",
				"exception: " + e);
		}
	}
      
      this.start();
    }
  
  public void run()
    {
      TypeReader typeReader;
      Message m;
      FSMMapInfo info;

      do
	{
	  typeReader = tsf_.createTypeReader(reader_);
	  try
	    {
	      m = factory_.createFromStream(typeReader);
                      if(Verbose.verbose)
			{
			  Verbose.print("SocketReader",
					"run",
					"read a message: " + m);
			}
	    }
	  catch(Exception e)
	    {
	      m = null;
                      if(Verbose.verbose)
			{
			  Verbose.print("SocketReader",
					"run",
					"exception: " + e);
			}
	    }

	  // retrieve the correct FSM cell
	  // if fsmId_ < 0 then this socket is dedicated to one FSM
	  // otherwise put it to the transaction id's cell
	  info = null;
	  if(fsmId_ < 0)
	    {
	      info = (FSMMapInfo)fsms_.get(new Integer(fsmId_));
                      if(Verbose.verbose)
			{
			  Verbose.print("SocketReader",
					"run",
					"got an FSMMapInfo with key: " +
					fsmId_);
			}
	    }
	  else
	    {
	      if(m != null)
		{		  
		  info =(FSMMapInfo)fsms_.get(new Integer(m.getTransactionId()));
                          if(Verbose.verbose)
			    {
			      Verbose.print("SocketReader",
					    "run",
					    "got an FSMMapInfo with key: " +
					    m.getTransactionId());
			    }
		}	      
	    }

	  if(info == null)
	    {
	      if(m != null)
		{
		  System.out.println("@SocketReader.run(): No destination for this message: " + m);
		}
	      else
		{
                          if(Verbose.verbose)
			    {
			      Verbose.print("SocketReader",
					    "run",
					    "both 'info' and 'm' are null");
			    }
		}
	    }
	  else
	    {
                          if(Verbose.verbose)
			    {
			      Verbose.print("SocketReader",
					    "run",
					    "set the message in 'info' for reception");
			    }
	      info.setMessage(m);
	      info.socket_ = (m == null)?null:socket_;
	    }

	}
      while(m != null);
      
                          if(Verbose.verbose)
			    {
			      Verbose.print("SocketReader",
					    "run",
					    "got a null message, shut down this socket");
			    }
      sh_.removeSocket(socket_.socket_);
    }
}
