package EDU.rutgers.cbus.clntsrv;

import EDU.rutgers.act.endpoint.Endpoint;
import EDU.rutgers.act.protocol.*;
import EDU.rutgers.act.protocol.tcp.*;
import EDU.rutgers.cbus.protocol.messages.CBusEvent;

public class EndpointManager
{
  protected Endpoint[] endpoints_;
  protected int count_;
  Sender clientSender_;
  Receiver clientReceiver_;
  boolean mcast_;

  public EndpointManager(Sender clientSender, boolean mcast)
    {
      count_ = 0;
      endpoints_ = new Endpoint[64];
      clientSender_ = clientSender;
      clientReceiver_ = null;
      mcast_ = mcast;
    }
  
  public EndpointManager(int size)
    {
      count_ = 0;
      endpoints_ = new Endpoint[size];
    }
  
  public synchronized void addEndpoint(Endpoint ep)
    {
      endpoints_[count_++] = ep;
    }
  
  public synchronized void removeEndpoint(Endpoint ep)
    {
      int i;
      for(i = 0; (i < count_) && !endpoints_[i].equals(ep); i++);
      if(i < count_)
	{
	  endpoints_[i] = null;
	  endpoints_[i] = endpoints_[--count_];  // fill the empty space
	}
      else
	System.out.println("EndpointManager.removeEndpoint(): socket not in my set");
    }

  public void replicateEvent(CBusEvent event)
    {
      CBusEvent clone;
      for(int i = 0; i < count_; i++)
	{
	  clone = (CBusEvent)event.clone();
	  clone.setEndpoint(endpoints_[i]);

	  try
	    {
	      clientSender_.sendMessage(clone);
	    }
	  catch(Exception e)
	    {
	      System.out.println("@EndpointManager: " + e);
	    }

	  if(!mcast_)
	    {
	      if(clientReceiver_ == null)
		clientReceiver_ = new ClientReceiver(event.getTransactionId());
	      try
		{
		  clientReceiver_.receiveMessage();
		}
	      catch(Exception e)
		{
		  System.out.println("@EndpointManager: " + e);
		}
	    }
	}
    }
}
