package EDU.rutgers.cbus.presentation;

import java.util.EventObject;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;
import java.awt.Component;

public class ComponentAdaptor extends EventAdaptor implements ComponentListener
{
  public ComponentAdaptor(Component component)
    {
      super(component);
    }
  
 /*******************************************************
  * The ComponentAdaptor is a source of ComponentEvents.
  * Therefore implement all needed methods.
  *******************************************************/
  public synchronized void addComponentListener(ComponentListener l)
    {
      eventListeners_.addElement(l);
    }
  
  public synchronized void removeComponentListener(ComponentListener l)
    {
      eventListeners_.removeElement(l);
    }

  protected void invoke(Object target, EventObject event)
    {
      ComponentEvent me = (ComponentEvent)event;
      ComponentListener ml = (ComponentListener) target;

      switch(me.getID())
	{
	case ComponentEvent.COMPONENT_HIDDEN:
	  ml.componentHidden(me);
	  break;
	case ComponentEvent.COMPONENT_MOVED:
	  ml.componentMoved(me);
	  break;
	case ComponentEvent.COMPONENT_RESIZED:
	  ml.componentResized(me);
	  break;
	case ComponentEvent.COMPONENT_SHOWN:
	  ml.componentShown(me);
	  break;
	default:
	  System.out.println("@ComponentAdaptor.invoke(): illegal ComponentEvent type");
	}
    }
 /*******************************************************
  * The ComponentAdaptor is a listener of ComponentEvents.
  * Therefore implement all needed methods.
  *******************************************************/
  public void componentHidden(ComponentEvent e)
    {
      this.processEvent(e);
    }
  
  public void componentMoved(ComponentEvent e)
    {
      this.processEvent(e);
    }

  public void componentResized(ComponentEvent e)
    {
      this.processEvent(e);
    }

  public void componentShown(ComponentEvent e)
    {
      this.processEvent(e);
    }
}
