package EDU.rutgers.cbus.presentation;

import java.awt.Component;
import java.awt.AWTEvent;
import java.util.EventListener;
import java.util.EventObject;
import java.util.Vector;

import EDU.rutgers.cbus.protocol.messages.CBusEvent;
import EDU.rutgers.cbus.protocol.messages.CBusEventListener;

public abstract class EventAdaptor
  extends CBusSource
  implements CBusEventListener
{
  protected static int testCounter_ = 0;

  protected Vector eventListeners_;

  protected Component component_;

  public EventAdaptor()
    {
      eventListeners_ = new Vector();
      component_ = null;
    }
  
  public EventAdaptor(Component component)
    {
      eventListeners_ = new Vector();
      component_ = component;
    }
  

 /*******************************************************
  * This EventObject can be any event, and comes from the
  * application itself through an event adapter generated
  * by the visual tool.
  *******************************************************/
  public void processEvent(EventObject event)
    {
      if(event.getSource() != null)
	{
	  CBusEvent cbev = new CBusEvent(0,
					 event);
	  fireCBusEvent(cbev);
	}
    }

 /*******************************************************
  * The EventAdaptor is a listener of CBusEvents.
  * Therefore implement all needed methods.
  *******************************************************/
  public void processCBusEvent(CBusEvent event)
    {
      fireEvent(event.getUserEvent());
    }

 /*******************************************************
  * The EventAdaptor is a source of Events.
  * Therefore implement all needed methods.
  *******************************************************/
  public void fireEvent(EventObject event)
    {
      Vector targets;
      synchronized (this)
	{
	  targets = (Vector) eventListeners_.clone();
	}

      for (int i = 0; i < targets.size(); i++)
	{
	  invoke(targets.elementAt(i), event);
	}

      if(component_ != null)
	{
//	  System.out.println("@EventAdaptor: dispatching event!");
	  component_.dispatchEvent((AWTEvent)event);
	}
    }

  public void setComponent(Component c)
    {
      component_ = c;
    }

  protected abstract void invoke(Object target, EventObject event);
}
