/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ROCS %Version%
 *
 */

package EDU.rutgers.util;

import java.util.Hashtable;

import java.util.StringTokenizer;

public class GetOpt
{
    /** 
     * @param args command line arguments
     * @param optstring string containing the legitimate option
     * characters. A colon in optstring means that the previous character
     * is an option that wants an argument which is then taken from the
     * rest of the current args-element. Here is an example of what
     * optstring might look like: "c:dP:p".
     */
    public GetOpt (String[] args, String optstring) {
        args_ = args;
        hasArg_ = false;
        buildArgTable(optstring);
    }

    /** 
     * Scan elements specified in optstring for next option flag.
     *
     * @return The character corresponding to the next flag.
     */
    public int next() {
        if (args_ == null)
            return -1;    
        if (index_ < args_.length) {
        	String arg = args_[index_++];
    	    if (arg.startsWith("-") == false)
                return -1;
    	    if (arg.length() > 2)
                return -1;	
    	    Character c = (Character)argTable_.get(new Character(arg.charAt (1)));
    	    if (c == null)
                return -1;
    	    if (c.charValue() == '#') {
        	    hasArg_ = false;
    	        return arg.charAt (1);
	        }
    	    else if (c.charValue () == ':') {
    	        hasArg_ = true;
    	        return arg.charAt (1); 
	        }
            else {
    	        return -1; 
            }
        }
        return -1;
    } 

    /**
     * Get the argument (if any) associated with the flag.
     *
     * @return the argument associated with the flag.
     */
    public String optarg () {
        if (hasArg_)
          return args_[index_++];
        else
          return null;
    }

    private void buildArgTable(String s) {
        argTable_ = new Hashtable();
        StringTokenizer tokens = new StringTokenizer(s, ":");
        while (tokens.hasMoreTokens()) {
        	String t = tokens.nextToken();
	        // First add all flags except the one with ":" after it
	        // Note "#" is an arbitrary character we use to distinguish
	        // the two cases
            for (int i = 0; i < t.length() - 1; i++) {    
	            argTable_.put(new Character(t.charAt(i)), new Character('#'));
            }
	        // Now Add the flag just before ":" to the arg_table
        	argTable_.put(new Character(t.charAt(t.length() - 1)), new Character(':'));
        }
    }

    private String [] args_;
    private boolean hasArg_;
    private int index_;
    private Hashtable argTable_;
}

