/*
 * @(#)%Name% %Version% %Time%
 * 
 * Copyright (c) 1997 Rutgers University
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that (i) the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation. (ii) 
 * the name of Rutgers University may not be used in any advertising
 * or publicity relating to the software without the specific, prior
 * written permission of Rutgers University.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL RUTGERS UNIVERSITY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * CopyrightVersion %Version%
 * @author		%Author%
 * @version		%Version%, %Time%
 * @since		ACT %Version%
 *
 */

package ORG.omg.CORBA;

import java.io.IOException;

import EDU.rutgers.act.presentation.TypeReader;
import EDU.rutgers.act.presentation.TypeWriter;

/**
 * Representation of bounded sequence<octet>. Sequence list should use java array.
 *
 * @since ACT 0.1
 */
public final class OctetSequence implements Streamable
{
    private byte[] octects_ = EMPTYByteArray.EMPTY; 

    /**
     * Used by Streamable
     *
     * @see Streamable
     *
     * @since ACT 0.1
     */
    public OctetSequence() {
    }

    /**
     * @since ACT 0.1
     */
    public OctetSequence(byte[] c) {
        if (c == null) {
            return; 
        }
        octects_ = c; 
    }

    /**
     * Retrieve the octet array
     *
     * @since ACT 0.1
     */
    public byte[] toBytes() {
        return octects_; 
    }

    /**
     * Retrieve sequence bounded length
     *
     * @since ACT 0.1
     */
    public int length() {
        return octects_.length; 
    }

    /**
     * @see Streamable
     * @see Streamable#read
     *
     * @since ACT 0.1
     */
    public void read(TypeReader in) 
		throws java.io.IOException {
        octects_ = new byte[in.readLong()]; 
        in.readOctetArray(octects_, 0, octects_.length); 
    }

    /**
     * @see Streamable
     * @see Streamable#write
     *
     * @since ACT 0.1
     */
    public void write(TypeWriter out) 
		throws java.io.IOException {
        out.writeLong(octects_.length); 
        out.writeOctetArray(octects_, 0, octects_.length); 
    }

    /**
     * Test the contents of two OctetSequence's equality
     *
     * @param Object another OctetSequence
     * @return boolean true if byte arrays are same, false if not
     *
     * @since ACT 0.1
     */
    public boolean equals(java.lang.Object o) {
        if((o != null) && (o instanceof OctetSequence)) {
			OctetSequence seq = (OctetSequence)o;
			String s1 = new String(octects_);
			String s2 = new String(seq.toBytes());
			return s1.equals(s2);
        }
        return false; 
    }

    /**
     * Returns a string representation.
     *
     * @return String description
     *
     * @since ACT 0.1
     */
    public String toString() {
        return OctetSequence.byteArray2String(octects_);
    }

    /**
     * Convert a byte[] to a readable string. 
     * The user could use this method to dump the octet array.
     *
     * @return String description
     *
     * @since ACT 0.1
     */
    public final static String byteArray2String(byte[] ba) {
        if (ba == null) {
            return "{null}";
        }
		int len = ba.length;
        if (len == 0) {
            return "{EMPTY}"; 
        }
        StringBuffer strBuf = (new StringBuffer("{")).append(len).append(" bytes: "); 
        for (int index = 0; index < len; index++) {
            int val = ba[index]; 
            if (val < 0x20 || val > 0x7E) {
                strBuf.append("(").append(String.valueOf(val)).append(")"); 
                continue;
            }
            strBuf.append("[").append((char)val).append("]"); 
        }
        return String.valueOf(strBuf) + "}";
    }

    /**
     * Dump the byte in HEX to the System.err
     *
     * @since ACT 0.1
     */
    public final static void dumpHex(byte b) {
		int n1 = b / 16;
		int n2 = b % 16;
		char c1 = (char)(n1 > 9 ? ('A' + (n1 - 10)) : ('0' + n1));
		char c2 = (char)(n2 > 9 ? ('A' + (n2 - 10)) : ('0' + n2));
		char c3;
		if (b > (byte)0x1F && b < (byte)0x7F)
			c3 = (char)b;
		else
			c3 = ' ';
		java.lang.System.out.print(c1 + (c2 +" ") + c3 +"  ");
	}

    /**
     * Dump the byte array in HEX to the System.err
     *
     * @since ACT 0.1
     */
	public final static void dumpBA(byte ba[]) {
		int len = ba.length;
        for (int i = len; i-- != 0;) {
			if (0 == i % 20) {
				java.lang.System.out.println();
			}
			dumpHex(ba[i]);
		}
		java.lang.System.out.println();
	}
}

