// last modified 12/18/97

package examples.ACTMUD;

import java.io.*;
import java.lang.*;
import examples.ACTMUD.generated.*;

public class _ACTMUDClntImpl extends _ACTMUDClntImplBase
{
  // constants
  protected final int CMD_NONE = -1;
  protected final int CMD_SAY  = 0;
  protected final int CMD_HELP = 1;
  protected final int CMD_QUIT = 2;
  protected final int CMD_LOOK = 3;

  protected final String[] commandNames = 
    {
      ".say",
      ".help",
      ".quit",
      ".look"
    };
  

  // data members
  InputStream in_;               // the non blocking input stream
  String name_;                  // name of the user
  ACTMUDSrv srvRef_;             // server stub

  public _ACTMUDClntImpl()
    {
      in_ = new NonblockingInputStream(System.in);
    }
 
  //
  // public methods (idl description)
  //
  public void type(String msg)
    {
      System.err.println(msg);
      System.err.flush();
    }
  
  //
  // local methods
  //
  protected void readName()
    {
      byte[] nameBytes = new byte[20];
      short length = 0;
      
      System.out.println("*****************************");
      System.out.println("****  Welcome to ACTMUD  ****");
      System.out.println("*****************************");
      System.out.println("What is your name?");
      
      // read the user name
      try
	{
	  length = (byte)in_.read(nameBytes);
	}
      catch (IOException e)
	{
	  System.out.println("Name error:" + e);
	}
      
      name_ = new String(nameBytes, 0, length - 1);
    }
  

  public void launch(ACTMUDSrv srvRef /*, ACTMUDClnt clntRef*/)
    {
      short cmdLen = 0;                // the command length
      byte[] command = new byte[1000]; //the command 
      
      // set the server reference
      srvRef_ = srvRef;

      // read user's name in variable name_
      readName();
      
      // register with the ACTMUD server
      srvRef.register(this, name_);
      
      System.out.println("Welcome " + name_);
      System.out.println("If you need help just type .help");
      
      System.err.println("You can enter commands now:");
      
      // enter the loop: read a command, execute it!
      do
	{
	  System.out.print("> ");
	  try
	    {
	      cmdLen = (byte)in_.read(command);
	    }
	  catch (IOException e)
	    {
	      System.out.println("Name error:" +e);
	    }
	}
      while(executeCmd(command, cmdLen));
    }

  //
  // returns the command code in cmd[0] and the command body
  //
  String split(byte[] command, short cmdLen, int[] cmd)
    {
      int i, len;
      for(i = 0; i < commandNames.length; i++)
	{
	  if (new String(command, 0, commandNames[i].length()).compareTo(
                         commandNames[i]) == 0)
	    {
	      cmd[0] = i;
	      len = cmdLen - commandNames[i].length() - 2;
	      return new String(command,
				commandNames[i].length() + 1, 
				len <= 0 ? 0 : len);
	    }
	}

      // not a valid command
      cmd[0] = CMD_NONE;
      return "";
    }
  

  protected boolean executeCmd(byte[] command, short cmdLen)
    {
      byte[] bytes = new byte[1000];
      int[] cmd = new int[1];

      String cmdBody = split(command, cmdLen, cmd);
      
      switch(cmd[0])
	{
	  // if the message typed is not a command type a error message
        case CMD_NONE:
	  System.out.println("What? Please type a command");
	  return true;
	  
	case CMD_QUIT:
	  System.out.println("Leaving the game... later.");

	  // call unregister method
	  srvRef_.unregister(name_);
	  return false;

	case CMD_SAY:
	  // calling the say method on server
	  srvRef_.say(cmdBody, name_);
	  return true;

      
	case CMD_HELP:
	  // read from booklet.txt file the commands for using MUD
	  try 
	    {
	      // open the file booklet.txt
	      FileInputStream file = new FileInputStream("booklet.txt");
	      
	      int fileLength;
	      fileLength = file.read(bytes);
	      while(fileLength >= 0)
		{
		  // read and display the file  
		  System.out.print(new String(bytes, 0, fileLength));
		  fileLength = file.read(bytes);
		}
	      // close the file
	      file.close();
	    } 
	  catch (FileNotFoundException e) 
	    {
	      System.err.println("FileStreamsTest: " + e);
	    } 
	  catch (IOException e) 
	    {
	      System.err.println("FileStreamsTest: " + e);
	    }
	  return true;

	case CMD_LOOK:
	  if (new String(command, 0, 5).compareTo(".look") == 0)
	    {
	      int i,j;
	      int MAX_LINE = 20;
	      int MAX_COL = 20;
	      int[][] map = new int [MAX_LINE][MAX_COL];
	      /*for (i=0; i<MAX_LINE, i++)
		{
		map[i] = new map*/
	      
	      // every element of this matrix is a "room" on the map
	      // every "room" is a file roomij.txt
	      // read from the file the description of the "room"
	      
	      try
		{
		  //decide what file to display
		  i = 0;
		  j = 0;
		  
		  // open the file booklet.txt
		  FileInputStream file = new FileInputStream("room" + i + j +".txt");
		  
		  int fileLength;
		  fileLength = file.read();
		  while(fileLength >= 0)
		    {
		      // read and display the file  
		      System.out.print(new String(bytes, 0, fileLength));
		      fileLength = file.read(bytes);
		    }
		  //close the file
		  file.close();
		} 
	      catch (FileNotFoundException e) 
		{
		  System.err.println("FileStreamsTest: " + e);
		} 
	      catch (IOException e) 
		{
		  System.err.println("FileStreamsTest: " + e);
		}
	    }
	  return true;
	  
	  
	  // if Commands are .south, .east, .west., .nord the posLin and posCol
	  // are modified
	  //
	}      
      return true;
    }
}

