// for emacs only: -*- C++ -*-

// last modification 12/15/97

import java.io.*;

import ORG.omg.CORBA.Principal;
import EDU.rutgers.act.endpoint.Endpoint;
import EDU.rutgers.act.protocol.*;
import EDU.rutgers.act.protocol.tcp.*;
import EDU.rutgers.act.presentation.*;
import EDU.rutgers.CORBA.presentation.*;
import EDU.rutgers.CORBA.GIOP.actions.*;
import EDU.rutgers.CORBA.GIOP.messages.*;
import EDU.rutgers.CORBA.IIOP.*;
import EDU.rutgers.CORBA.IOP.*;

class Server
{
  public static void main(String args[]) throws java.io.IOException
    {
      if(args.length != 1)
	{
	  System.out.println("usage: Server <portnumber>");
	  throw new IOException("Invalid arguments");
	}

      FSMDataFactory giopSrvFactory =
	(FSMDataFactory)new GIOPServerDataFactory();
      MessageFactory msgFactory = 
	(MessageFactory)new GIOPMessageFactory();

      ProtocolServer protSrv =
	new ProtocolServer(giopSrvFactory,
			   msgFactory,
			   new CDRTransferSyntaxFactory(),
			   Integer.parseInt(args[0]));

      while(true)
	{
	  Protocol protocol = protSrv.accept();

	  GIOPRequestMessage request;

	  request = 
	    (GIOPRequestMessage)protocol.getServerReceiver().receiveMessage();

	  while(request != null)
	    {
	      if(!"01".equals(new String(request.object_key_)))
		{
		  System.out.println("Object not here: " + new String(request.object_key_));
		}
	      else
		{
		  //
		  // create the type reader for getting the content of the body
		  //
		  ByteArrayInputStream stream = 
		    new ByteArrayInputStream(request.getRequestBody());
		  TypeReader in = 
		    new CDRTypeReader(stream);

		  in.setIndex(request.getBeforeBodySize());
		  in.setByteOrder(request.byte_order_);


		  //
		  // read the data inside the body of the Request message
		  //
		  String arg = "";
		  try {
		    arg = in.readString();
		    // ...
		    // read primitive types of data using TypeReader primitives
		  }
		  catch(java.io.IOException e)
		    {
		      System.out.println("Marshaling/unmarshaling error: " + e);
		      System.exit(1);
		}

		  //		  System.out.print("*");


		  // compose the reply and open the output stream (Reply Body)
		  GIOPReplyMessage reply = 
		    new GIOPReplyMessage(request.service_context_, 
			   request.request_id_,
			   ReplyStatusType.NO_EXCEPTION);

		  ByteArrayOutputStream outStream = new ByteArrayOutputStream();
		  CDRTypeWriter out = 
		    new CDRTypeWriter(outStream);
		  // to byte[]
		  out.setByteOrder(reply.byte_order_);
		  out.setIndex(reply.getBeforeBodySize());

		  out.writeString(arg);
		  out.flush();
		  byte[] replyBody = outStream.toByteArray();
	
		  // set the reply body
		  reply.setReplyBody(replyBody);
            
		  if(protocol.getServerSender() == null)
		    {
		      System.out.println("Null!");
		    }
	  
		  protocol.getServerSender().sendMessage((Message)reply);
		}
	      request = 
		(GIOPRequestMessage)protocol.getServerReceiver().receiveMessage();
	    }
	  System.out.println("Received a null message!");
	}
    }

  static protected String revert(String inStr)
    {
      byte[] in = inStr.getBytes();
      byte[] out = new byte[in.length];
      for(int i = 0; i < in.length; i++)
	{
	  out[i] = in[in.length - i -1];
	}
      return new String(out);
    }
}
