// for emacs only: -*- C++ -*-

// last modification 12/13/97

package examples.TSTP.messages;

import java.io.*;
import EDU.rutgers.act.protocol.Message;
import EDU.rutgers.act.endpoint.Endpoint;
import EDU.rutgers.act.presentation.TypeWriter;
import EDU.rutgers.act.presentation.TypeReader;

public class TSTPMessage implements Message
{
  // constants

  final static public int Request = 0;
  final static public int Reply = 1;

  final static public int ECHO = 0;
  final static public int REVERT = 1;
  final static public int TOUPPER = 2;
  final static public int TOLOWER = 3;

  // data members

  protected int messageType_;
  protected int opId_;
  protected int length_;
  protected String string_;
  protected Endpoint destination_;

  // constructors

  // used by the factory
  public TSTPMessage()
    {
    }

  // used by the application programmer
  public TSTPMessage(int messageType,
		     int opId,
		     String string,
		     Endpoint destination)
    {
      messageType_ = messageType;
      opId_ = opId;
      string_ = new String(string);
      length_ = string.length();
      destination_ = destination;
    }
  
  public int getType()
    {
      return messageType_;
    }
  
  public void write(TypeWriter os) throws IOException
    {
      byte[] buffer;
      if(messageType_ == 0)
	{
	  buffer = new byte[4];
	  buffer[0] = (byte)messageType_;
	  buffer[1] = (byte)opId_;
	  buffer[2] = (byte)length_;
	  buffer[3] = (byte)(length_ >> 8);
	}
      else
	{
	  buffer = new byte[3];
	  buffer[0] = (byte)messageType_;
	  buffer[1] = (byte)length_;
	  buffer[2] = (byte)(length_ >> 8);
	}

      os.writeOctetArray(buffer, 0, buffer.length);
      if(length_ >= 0)
	{
	  os.writeOctetArray(string_.getBytes(), 0, string_.length());
	}
    }
  
  public void read(TypeReader is) throws IOException
    {
      messageType_ = is.read();
      if(messageType_ == TSTPMessage.Request)
	{
	  opId_ = is.read();
	}
      short len1 = (short)is.read();
      length_ = (int)((((short)is.read()) << 8) | len1);

      if(length_ > 0)
	{
	  byte[] str = new byte[length_];
	  is.readOctetArray(str, 0, str.length);
	  string_ = new String(str);
	}

      destination_ = null;
    }

  // accessors

  public int getOpId()
    {
      return opId_;
    }
  public int getLength()
    {
      return length_;
    }
  public String getString()
    {
      return string_;
    }
  public Endpoint getEndpoint()
    {
      return destination_;
    }
  public int getTransactionId()
    {
      return 0;
    }
}
