package examples.milmap;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.io.Serializable;

public class MilMap extends Panel implements MouseListener,
	MouseMotionListener, Serializable
{
  private static final String MAP_FILE = "map4.gif";
  private static final String BASE_DIR = 
    "./";
  private transient Image image_;
  private transient Image offScrImg_;
  private Vector icons_;
  private Icon hitIcon_;
  private int width_ = 966;
  private int height_ = 750;
  private String[] iconName_;

  /** 
   * Initialize the MilMap panel.
   */

  public MilMap()
    {
      setLayout(null);
      setSize(500, 400);

      // Load the map...
      image_ = loadImage(MAP_FILE);

      //load the icons...
      icons_ = new Vector();
      iconName_ = new String[4];

      iconName_[0] = "circle_dot.gif";
      iconName_[1] = "circle_dot3.gif";
      iconName_[2] = "circle_line_dot.gif";
      iconName_[3] = "circle_line_dot2.gif";

      addIcon(iconName_[0]);
      addIcon(iconName_[1]);
      addIcon(iconName_[2]);
      addIcon(iconName_[3]);

      addMouseMotionListener(this);
      addMouseListener(this);
      repaint();
    }

  /**
   * This is an internal utility method to load GIF icons.
   * It takes the name of a resource file associated with the
   * current object's class-loader and loads a GIF image
   * from that file.
   * <p>
   * @param resourceName    A pathname relative to the DocumentBase
   *		of this applet, e.g. "wombat.gif".
   * @return    a GIF image object.    May be null if the load failed.
   */
  private Image loadImage(String name)
    {
      try
	{
	  Image img = this.getToolkit().getImage(BASE_DIR + name);
	  if(img == null)
	    {
	      System.out.println("can't load " + "/u/francu/disciple/java/examples/sunw/demo/militarytrans/" + name);
	    }
	  else
	    this.waitImage(img);
	
	  return img;
	}
      catch (Exception ex)
	{
	  System.out.println("can't load " + "/caip/u4/francu/cbus/java/examples/sunw/demo/militarytrans/" + name);
	  return null;
	}
    }

  public void addIcon(String iconName)
    {
      Image img = loadImage(iconName);
      Icon icon = new Icon(img, 100, 100);
      icons_.addElement(icon);
      repaint();
    }
  
  public void update(Graphics gc)
    {
      paint(gc);
    }

  public void paint(Graphics gc)
    {
      if(offScrImg_ == null)
	{
	  offScrImg_ = createImage(width_, height_);
	}
		
      Graphics og = offScrImg_.getGraphics();

      //draw map
      if(image_ == null)
	image_ = loadImage(MAP_FILE);	
      og.drawImage(image_, 0, 0, null);

      //draw icons
      int i;
      int numIcons = icons_.size();
      Icon icon;
      for(i = 0; i < numIcons; i++)
	{
	  icon = (Icon) icons_.elementAt(i);
	  if(icon.getIconImage() == null)
	    {
	      Image img = loadImage(iconName_[i]);
	      icon.setIconImage(img);
	    }
	  og.drawImage(icon.getIconImage(), icon.getX(),
		       icon.getY(), null);
	}

      gc.drawImage(offScrImg_, 0, 0, this);
      og.dispose();
    }

  /** 
   * If false, suspend the animation thread.
   */
  public synchronized void setEnabled(boolean x)
    {
      super.setEnabled(x);
      notify();
    }	

  public Dimension getMinimumSize()
    {
      return new Dimension(width_, height_);
    }

  /**
   * @deprecated provided for backward compatibility
   *  with old layout managers.
   */
  public Dimension minimumSize()
    {
      return getMinimumSize();
    }

  //-----------------------------------------
  //Mouse Motion Listener methods
  
  public void mouseDragged(MouseEvent e)
    {
      Point p = e.getPoint();
      if(hitIcon_ != null)
	{
	  hitIcon_.setPosition(p);
	  repaint();
	}
    }

  public void mouseMoved(MouseEvent e) {}

  //-----------------------------------------
  //Mouse Listener methods

  public void mousePressed(MouseEvent e)
    {
      Point p = e.getPoint();
      System.out.println("mouse pressed");
      hitIcon_ = map(p);
    }
  
  public void mouseReleased(MouseEvent e) {}
  
  public void mouseClicked(MouseEvent e) {}

  public void mouseEntered(MouseEvent e) {}

  public void mouseExited(MouseEvent e) {}

  private Icon map(Point p)
    {
      Icon icon;
      int numIcons = icons_.size();	
      int i;	
      for(i = numIcons - 1; i >= 0; i--)
	{
	  icon = (Icon) icons_.elementAt(i);
	  if(p.x >= icon.getX() && p.x <= icon.getX() + icon.getWidth() &&
	     p.y >= icon.getY() && p.y <= icon.getY() + icon.getHeight())
	    {
	      System.out.println("hit icon #" + i);
	      return icon;
	    }
	}
      
      return null;
    }

  public boolean waitImage(Image image)
    {
      if(image == null) return false;
      MediaTracker tracker = new MediaTracker(this);
      tracker.addImage(image, 0);
      try
        {
          tracker.waitForID(0);
        }
      catch(InterruptedException e) {}
      if (tracker.isErrorAny()) {
        System.err.println("Error loading image: "+ image);
        return false;
      }
      return true;
    }
}
